@@ -86,7 +86,7 @@
 #include "charset.h"
 #include "block.h"
 
-#define parse_type builtin_type (parse_gdbarch)
+#define parse_type(ps) builtin_type (parse_gdbarch (ps))
 
 /* Remap normal yacc parser interface names (yyparse, yylex, yyerror, etc),
    as well as gratuitiously global symbol names, so we can have multiple
@@ -142,6 +142,11 @@
 
 #define YYFPRINTF parser_fprintf
 
+/* The state of the parser, used internally when we are parsing the
+   expression.  */
+
+static struct parser_state *pstate = NULL;
+
 int yyparse (void);
 
 static int yylex (void);
@@ -151,7 +156,7 @@ void yyerror (char *);
 
 
 /* Line 189 of yacc.c  */
-#line 155 "go-exp.c"
+#line 160 "go-exp.c"
 
 /* Enabling traces.  */
 #ifndef YYDEBUG
@@ -274,7 +279,7 @@ typedef union YYSTYPE
 {
 
 /* Line 214 of yacc.c  */
-#line 138 "go-exp.y"
+#line 143 "go-exp.y"
 
     LONGEST lval;
     struct {
@@ -298,7 +303,7 @@ typedef union YYSTYPE
 
 
 /* Line 214 of yacc.c  */
-#line 302 "go-exp.c"
+#line 307 "go-exp.c"
 } YYSTYPE;
 # define YYSTYPE_IS_TRIVIAL 1
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
@@ -309,16 +314,17 @@ typedef union YYSTYPE
 /* Copy the second part of user declarations.  */
 
 /* Line 264 of yacc.c  */
-#line 159 "go-exp.y"
+#line 164 "go-exp.y"
 
 /* YYSTYPE gets defined by %union.  */
-static int parse_number (const char *, int, int, YYSTYPE *);
+static int parse_number (struct parser_state *,
+			 const char *, int, int, YYSTYPE *);
 static int parse_go_float (struct gdbarch *gdbarch, const char *p, int len,
 			   DOUBLEST *d, struct type **t);
 
 
 /* Line 264 of yacc.c  */
-#line 322 "go-exp.c"
+#line 328 "go-exp.c"
 
 #ifdef short
 # undef short
@@ -630,13 +636,13 @@ static const yytype_int8 yyrhs[] =
 /* YYRLINE[YYN] -- source line where rule number YYN was defined.  */
 static const yytype_uint16 yyrline[] =
 {
-       0,   237,   237,   238,   241,   248,   249,   254,   258,   262,
-     266,   270,   274,   278,   282,   288,   294,   301,   311,   318,
-     315,   325,   329,   332,   336,   340,   344,   350,   356,   362,
-     366,   370,   374,   378,   382,   386,   390,   394,   398,   402,
-     406,   410,   414,   418,   422,   426,   430,   434,   438,   442,
-     446,   452,   459,   468,   481,   488,   491,   497,   509,   516,
-     533,   551,   562,   568,   574,   590,   645,   647,   654,   667
+       0,   243,   243,   244,   247,   254,   255,   260,   264,   268,
+     272,   276,   280,   284,   288,   294,   300,   307,   317,   324,
+     321,   332,   336,   339,   343,   347,   351,   357,   363,   369,
+     373,   377,   381,   385,   389,   393,   397,   401,   405,   409,
+     413,   417,   421,   425,   429,   433,   437,   441,   445,   449,
+     453,   459,   466,   475,   488,   495,   498,   504,   519,   526,
+     543,   561,   573,   579,   585,   601,   656,   658,   665,   678
 };
 #endif
 
@@ -1687,424 +1693,428 @@ yyreduce:
         case 4:
 
 /* Line 1455 of yacc.c  */
-#line 242 "go-exp.y"
-    { write_exp_elt_opcode(OP_TYPE);
-			  write_exp_elt_type((yyvsp[(1) - (1)].tval));
-			  write_exp_elt_opcode(OP_TYPE); }
+#line 248 "go-exp.y"
+    { write_exp_elt_opcode (pstate, OP_TYPE);
+			  write_exp_elt_type (pstate, (yyvsp[(1) - (1)].tval));
+			  write_exp_elt_opcode (pstate, OP_TYPE); }
     break;
 
   case 6:
 
 /* Line 1455 of yacc.c  */
-#line 250 "go-exp.y"
-    { write_exp_elt_opcode (BINOP_COMMA); }
+#line 256 "go-exp.y"
+    { write_exp_elt_opcode (pstate, BINOP_COMMA); }
     break;
 
   case 7:
 
 /* Line 1455 of yacc.c  */
-#line 255 "go-exp.y"
-    { write_exp_elt_opcode (UNOP_IND); }
+#line 261 "go-exp.y"
+    { write_exp_elt_opcode (pstate, UNOP_IND); }
     break;
 
   case 8:
 
 /* Line 1455 of yacc.c  */
-#line 259 "go-exp.y"
-    { write_exp_elt_opcode (UNOP_ADDR); }
+#line 265 "go-exp.y"
+    { write_exp_elt_opcode (pstate, UNOP_ADDR); }
     break;
 
   case 9:
 
 /* Line 1455 of yacc.c  */
-#line 263 "go-exp.y"
-    { write_exp_elt_opcode (UNOP_NEG); }
+#line 269 "go-exp.y"
+    { write_exp_elt_opcode (pstate, UNOP_NEG); }
     break;
 
   case 10:
 
 /* Line 1455 of yacc.c  */
-#line 267 "go-exp.y"
-    { write_exp_elt_opcode (UNOP_PLUS); }
+#line 273 "go-exp.y"
+    { write_exp_elt_opcode (pstate, UNOP_PLUS); }
     break;
 
   case 11:
 
 /* Line 1455 of yacc.c  */
-#line 271 "go-exp.y"
-    { write_exp_elt_opcode (UNOP_LOGICAL_NOT); }
+#line 277 "go-exp.y"
+    { write_exp_elt_opcode (pstate, UNOP_LOGICAL_NOT); }
     break;
 
   case 12:
 
 /* Line 1455 of yacc.c  */
-#line 275 "go-exp.y"
-    { write_exp_elt_opcode (UNOP_COMPLEMENT); }
+#line 281 "go-exp.y"
+    { write_exp_elt_opcode (pstate, UNOP_COMPLEMENT); }
     break;
 
   case 13:
 
 /* Line 1455 of yacc.c  */
-#line 279 "go-exp.y"
-    { write_exp_elt_opcode (UNOP_POSTINCREMENT); }
+#line 285 "go-exp.y"
+    { write_exp_elt_opcode (pstate, UNOP_POSTINCREMENT); }
     break;
 
   case 14:
 
 /* Line 1455 of yacc.c  */
-#line 283 "go-exp.y"
-    { write_exp_elt_opcode (UNOP_POSTDECREMENT); }
+#line 289 "go-exp.y"
+    { write_exp_elt_opcode (pstate, UNOP_POSTDECREMENT); }
     break;
 
   case 15:
 
 /* Line 1455 of yacc.c  */
-#line 289 "go-exp.y"
-    { write_exp_elt_opcode (STRUCTOP_STRUCT);
-			  write_exp_string ((yyvsp[(3) - (3)].ssym).stoken);
-			  write_exp_elt_opcode (STRUCTOP_STRUCT); }
+#line 295 "go-exp.y"
+    { write_exp_elt_opcode (pstate, STRUCTOP_STRUCT);
+			  write_exp_string (pstate, (yyvsp[(3) - (3)].ssym).stoken);
+			  write_exp_elt_opcode (pstate, STRUCTOP_STRUCT); }
     break;
 
   case 16:
 
 /* Line 1455 of yacc.c  */
-#line 295 "go-exp.y"
-    { mark_struct_expression ();
-			  write_exp_elt_opcode (STRUCTOP_STRUCT);
-			  write_exp_string ((yyvsp[(3) - (4)].ssym).stoken);
-			  write_exp_elt_opcode (STRUCTOP_STRUCT); }
+#line 301 "go-exp.y"
+    { mark_struct_expression (pstate);
+			  write_exp_elt_opcode (pstate, STRUCTOP_STRUCT);
+			  write_exp_string (pstate, (yyvsp[(3) - (4)].ssym).stoken);
+			  write_exp_elt_opcode (pstate, STRUCTOP_STRUCT); }
     break;
 
   case 17:
 
 /* Line 1455 of yacc.c  */
-#line 302 "go-exp.y"
+#line 308 "go-exp.y"
     { struct stoken s;
-			  mark_struct_expression ();
-			  write_exp_elt_opcode (STRUCTOP_STRUCT);
+			  mark_struct_expression (pstate);
+			  write_exp_elt_opcode (pstate, STRUCTOP_STRUCT);
 			  s.ptr = "";
 			  s.length = 0;
-			  write_exp_string (s);
-			  write_exp_elt_opcode (STRUCTOP_STRUCT); }
+			  write_exp_string (pstate, s);
+			  write_exp_elt_opcode (pstate, STRUCTOP_STRUCT); }
     break;
 
   case 18:
 
 /* Line 1455 of yacc.c  */
-#line 312 "go-exp.y"
-    { write_exp_elt_opcode (BINOP_SUBSCRIPT); }
+#line 318 "go-exp.y"
+    { write_exp_elt_opcode (pstate, BINOP_SUBSCRIPT); }
     break;
 
   case 19:
 
 /* Line 1455 of yacc.c  */
-#line 318 "go-exp.y"
+#line 324 "go-exp.y"
     { start_arglist (); }
     break;
 
   case 20:
 
 /* Line 1455 of yacc.c  */
-#line 320 "go-exp.y"
-    { write_exp_elt_opcode (OP_FUNCALL);
-			  write_exp_elt_longcst ((LONGEST) end_arglist ());
-			  write_exp_elt_opcode (OP_FUNCALL); }
+#line 326 "go-exp.y"
+    { write_exp_elt_opcode (pstate, OP_FUNCALL);
+			  write_exp_elt_longcst (pstate,
+						 (LONGEST) end_arglist ());
+			  write_exp_elt_opcode (pstate, OP_FUNCALL); }
     break;
 
   case 21:
 
 /* Line 1455 of yacc.c  */
-#line 326 "go-exp.y"
+#line 333 "go-exp.y"
     { start_arglist (); }
     break;
 
   case 23:
 
 /* Line 1455 of yacc.c  */
-#line 333 "go-exp.y"
+#line 340 "go-exp.y"
     { arglist_len = 1; }
     break;
 
   case 24:
 
 /* Line 1455 of yacc.c  */
-#line 337 "go-exp.y"
+#line 344 "go-exp.y"
     { arglist_len++; }
     break;
 
   case 25:
 
 /* Line 1455 of yacc.c  */
-#line 341 "go-exp.y"
+#line 348 "go-exp.y"
     { (yyval.lval) = end_arglist () - 1; }
     break;
 
   case 26:
 
 /* Line 1455 of yacc.c  */
-#line 345 "go-exp.y"
-    { write_exp_elt_opcode (UNOP_MEMVAL);
-			  write_exp_elt_type ((yyvsp[(2) - (4)].tval));
-			  write_exp_elt_opcode (UNOP_MEMVAL); }
+#line 352 "go-exp.y"
+    { write_exp_elt_opcode (pstate, UNOP_MEMVAL);
+			  write_exp_elt_type (pstate, (yyvsp[(2) - (4)].tval));
+			  write_exp_elt_opcode (pstate, UNOP_MEMVAL); }
     break;
 
   case 27:
 
 /* Line 1455 of yacc.c  */
-#line 351 "go-exp.y"
-    { write_exp_elt_opcode (UNOP_CAST);
-			  write_exp_elt_type ((yyvsp[(1) - (4)].tval));
-			  write_exp_elt_opcode (UNOP_CAST); }
+#line 358 "go-exp.y"
+    { write_exp_elt_opcode (pstate, UNOP_CAST);
+			  write_exp_elt_type (pstate, (yyvsp[(1) - (4)].tval));
+			  write_exp_elt_opcode (pstate, UNOP_CAST); }
     break;
 
   case 28:
 
 /* Line 1455 of yacc.c  */
-#line 357 "go-exp.y"
+#line 364 "go-exp.y"
     { }
     break;
 
   case 29:
 
 /* Line 1455 of yacc.c  */
-#line 363 "go-exp.y"
-    { write_exp_elt_opcode (BINOP_REPEAT); }
+#line 370 "go-exp.y"
+    { write_exp_elt_opcode (pstate, BINOP_REPEAT); }
     break;
 
   case 30:
 
 /* Line 1455 of yacc.c  */
-#line 367 "go-exp.y"
-    { write_exp_elt_opcode (BINOP_MUL); }
+#line 374 "go-exp.y"
+    { write_exp_elt_opcode (pstate, BINOP_MUL); }
     break;
 
   case 31:
 
 /* Line 1455 of yacc.c  */
-#line 371 "go-exp.y"
-    { write_exp_elt_opcode (BINOP_DIV); }
+#line 378 "go-exp.y"
+    { write_exp_elt_opcode (pstate, BINOP_DIV); }
     break;
 
   case 32:
 
 /* Line 1455 of yacc.c  */
-#line 375 "go-exp.y"
-    { write_exp_elt_opcode (BINOP_REM); }
+#line 382 "go-exp.y"
+    { write_exp_elt_opcode (pstate, BINOP_REM); }
     break;
 
   case 33:
 
 /* Line 1455 of yacc.c  */
-#line 379 "go-exp.y"
-    { write_exp_elt_opcode (BINOP_ADD); }
+#line 386 "go-exp.y"
+    { write_exp_elt_opcode (pstate, BINOP_ADD); }
     break;
 
   case 34:
 
 /* Line 1455 of yacc.c  */
-#line 383 "go-exp.y"
-    { write_exp_elt_opcode (BINOP_SUB); }
+#line 390 "go-exp.y"
+    { write_exp_elt_opcode (pstate, BINOP_SUB); }
     break;
 
   case 35:
 
 /* Line 1455 of yacc.c  */
-#line 387 "go-exp.y"
-    { write_exp_elt_opcode (BINOP_LSH); }
+#line 394 "go-exp.y"
+    { write_exp_elt_opcode (pstate, BINOP_LSH); }
     break;
 
   case 36:
 
 /* Line 1455 of yacc.c  */
-#line 391 "go-exp.y"
-    { write_exp_elt_opcode (BINOP_RSH); }
+#line 398 "go-exp.y"
+    { write_exp_elt_opcode (pstate, BINOP_RSH); }
     break;
 
   case 37:
 
 /* Line 1455 of yacc.c  */
-#line 395 "go-exp.y"
-    { write_exp_elt_opcode (BINOP_EQUAL); }
+#line 402 "go-exp.y"
+    { write_exp_elt_opcode (pstate, BINOP_EQUAL); }
     break;
 
   case 38:
 
 /* Line 1455 of yacc.c  */
-#line 399 "go-exp.y"
-    { write_exp_elt_opcode (BINOP_NOTEQUAL); }
+#line 406 "go-exp.y"
+    { write_exp_elt_opcode (pstate, BINOP_NOTEQUAL); }
     break;
 
   case 39:
 
 /* Line 1455 of yacc.c  */
-#line 403 "go-exp.y"
-    { write_exp_elt_opcode (BINOP_LEQ); }
+#line 410 "go-exp.y"
+    { write_exp_elt_opcode (pstate, BINOP_LEQ); }
     break;
 
   case 40:
 
 /* Line 1455 of yacc.c  */
-#line 407 "go-exp.y"
-    { write_exp_elt_opcode (BINOP_GEQ); }
+#line 414 "go-exp.y"
+    { write_exp_elt_opcode (pstate, BINOP_GEQ); }
     break;
 
   case 41:
 
 /* Line 1455 of yacc.c  */
-#line 411 "go-exp.y"
-    { write_exp_elt_opcode (BINOP_LESS); }
+#line 418 "go-exp.y"
+    { write_exp_elt_opcode (pstate, BINOP_LESS); }
     break;
 
   case 42:
 
 /* Line 1455 of yacc.c  */
-#line 415 "go-exp.y"
-    { write_exp_elt_opcode (BINOP_GTR); }
+#line 422 "go-exp.y"
+    { write_exp_elt_opcode (pstate, BINOP_GTR); }
     break;
 
   case 43:
 
 /* Line 1455 of yacc.c  */
-#line 419 "go-exp.y"
-    { write_exp_elt_opcode (BINOP_BITWISE_AND); }
+#line 426 "go-exp.y"
+    { write_exp_elt_opcode (pstate, BINOP_BITWISE_AND); }
     break;
 
   case 44:
 
 /* Line 1455 of yacc.c  */
-#line 423 "go-exp.y"
-    { write_exp_elt_opcode (BINOP_BITWISE_XOR); }
+#line 430 "go-exp.y"
+    { write_exp_elt_opcode (pstate, BINOP_BITWISE_XOR); }
     break;
 
   case 45:
 
 /* Line 1455 of yacc.c  */
-#line 427 "go-exp.y"
-    { write_exp_elt_opcode (BINOP_BITWISE_IOR); }
+#line 434 "go-exp.y"
+    { write_exp_elt_opcode (pstate, BINOP_BITWISE_IOR); }
     break;
 
   case 46:
 
 /* Line 1455 of yacc.c  */
-#line 431 "go-exp.y"
-    { write_exp_elt_opcode (BINOP_LOGICAL_AND); }
+#line 438 "go-exp.y"
+    { write_exp_elt_opcode (pstate, BINOP_LOGICAL_AND); }
     break;
 
   case 47:
 
 /* Line 1455 of yacc.c  */
-#line 435 "go-exp.y"
-    { write_exp_elt_opcode (BINOP_LOGICAL_OR); }
+#line 442 "go-exp.y"
+    { write_exp_elt_opcode (pstate, BINOP_LOGICAL_OR); }
     break;
 
   case 48:
 
 /* Line 1455 of yacc.c  */
-#line 439 "go-exp.y"
-    { write_exp_elt_opcode (TERNOP_COND); }
+#line 446 "go-exp.y"
+    { write_exp_elt_opcode (pstate, TERNOP_COND); }
     break;
 
   case 49:
 
 /* Line 1455 of yacc.c  */
-#line 443 "go-exp.y"
-    { write_exp_elt_opcode (BINOP_ASSIGN); }
+#line 450 "go-exp.y"
+    { write_exp_elt_opcode (pstate, BINOP_ASSIGN); }
     break;
 
   case 50:
 
 /* Line 1455 of yacc.c  */
-#line 447 "go-exp.y"
-    { write_exp_elt_opcode (BINOP_ASSIGN_MODIFY);
-			  write_exp_elt_opcode ((yyvsp[(2) - (3)].opcode));
-			  write_exp_elt_opcode (BINOP_ASSIGN_MODIFY); }
+#line 454 "go-exp.y"
+    { write_exp_elt_opcode (pstate, BINOP_ASSIGN_MODIFY);
+			  write_exp_elt_opcode (pstate, (yyvsp[(2) - (3)].opcode));
+			  write_exp_elt_opcode (pstate, BINOP_ASSIGN_MODIFY); }
     break;
 
   case 51:
 
 /* Line 1455 of yacc.c  */
-#line 453 "go-exp.y"
-    { write_exp_elt_opcode (OP_LONG);
-			  write_exp_elt_type ((yyvsp[(1) - (1)].typed_val_int).type);
-			  write_exp_elt_longcst ((LONGEST)((yyvsp[(1) - (1)].typed_val_int).val));
-			  write_exp_elt_opcode (OP_LONG); }
+#line 460 "go-exp.y"
+    { write_exp_elt_opcode (pstate, OP_LONG);
+			  write_exp_elt_type (pstate, (yyvsp[(1) - (1)].typed_val_int).type);
+			  write_exp_elt_longcst (pstate, (LONGEST)((yyvsp[(1) - (1)].typed_val_int).val));
+			  write_exp_elt_opcode (pstate, OP_LONG); }
     break;
 
   case 52:
 
 /* Line 1455 of yacc.c  */
-#line 460 "go-exp.y"
+#line 467 "go-exp.y"
     {
 			  struct stoken_vector vec;
 			  vec.len = 1;
 			  vec.tokens = &(yyvsp[(1) - (1)].tsval);
-			  write_exp_string_vector ((yyvsp[(1) - (1)].tsval).type, &vec);
+			  write_exp_string_vector (pstate, (yyvsp[(1) - (1)].tsval).type, &vec);
 			}
     break;
 
   case 53:
 
 /* Line 1455 of yacc.c  */
-#line 469 "go-exp.y"
+#line 476 "go-exp.y"
     { YYSTYPE val;
-			  parse_number ((yyvsp[(1) - (1)].ssym).stoken.ptr, (yyvsp[(1) - (1)].ssym).stoken.length,
-					0, &val);
-			  write_exp_elt_opcode (OP_LONG);
-			  write_exp_elt_type (val.typed_val_int.type);
-			  write_exp_elt_longcst ((LONGEST)
+			  parse_number (pstate, (yyvsp[(1) - (1)].ssym).stoken.ptr,
+					(yyvsp[(1) - (1)].ssym).stoken.length, 0, &val);
+			  write_exp_elt_opcode (pstate, OP_LONG);
+			  write_exp_elt_type (pstate, val.typed_val_int.type);
+			  write_exp_elt_longcst (pstate, (LONGEST)
 						 val.typed_val_int.val);
-			  write_exp_elt_opcode (OP_LONG);
+			  write_exp_elt_opcode (pstate, OP_LONG);
 			}
     break;
 
   case 54:
 
 /* Line 1455 of yacc.c  */
-#line 482 "go-exp.y"
-    { write_exp_elt_opcode (OP_DOUBLE);
-			  write_exp_elt_type ((yyvsp[(1) - (1)].typed_val_float).type);
-			  write_exp_elt_dblcst ((yyvsp[(1) - (1)].typed_val_float).dval);
-			  write_exp_elt_opcode (OP_DOUBLE); }
+#line 489 "go-exp.y"
+    { write_exp_elt_opcode (pstate, OP_DOUBLE);
+			  write_exp_elt_type (pstate, (yyvsp[(1) - (1)].typed_val_float).type);
+			  write_exp_elt_dblcst (pstate, (yyvsp[(1) - (1)].typed_val_float).dval);
+			  write_exp_elt_opcode (pstate, OP_DOUBLE); }
     break;
 
   case 56:
 
 /* Line 1455 of yacc.c  */
-#line 492 "go-exp.y"
+#line 499 "go-exp.y"
     {
-			  write_dollar_variable ((yyvsp[(1) - (1)].sval));
+			  write_dollar_variable (pstate, (yyvsp[(1) - (1)].sval));
 			}
     break;
 
   case 57:
 
 /* Line 1455 of yacc.c  */
-#line 498 "go-exp.y"
+#line 505 "go-exp.y"
     {
 			  /* TODO(dje): Go objects in structs.  */
-			  write_exp_elt_opcode (OP_LONG);
+			  write_exp_elt_opcode (pstate, OP_LONG);
 			  /* TODO(dje): What's the right type here?  */
-			  write_exp_elt_type (parse_type->builtin_unsigned_int);
+			  write_exp_elt_type
+			    (pstate,
+			     parse_type (pstate)->builtin_unsigned_int);
 			  CHECK_TYPEDEF ((yyvsp[(3) - (4)].tval));
-			  write_exp_elt_longcst ((LONGEST) TYPE_LENGTH ((yyvsp[(3) - (4)].tval)));
-			  write_exp_elt_opcode (OP_LONG);
+			  write_exp_elt_longcst (pstate,
+						 (LONGEST) TYPE_LENGTH ((yyvsp[(3) - (4)].tval)));
+			  write_exp_elt_opcode (pstate, OP_LONG);
 			}
     break;
 
   case 58:
 
 /* Line 1455 of yacc.c  */
-#line 510 "go-exp.y"
+#line 520 "go-exp.y"
     {
 			  /* TODO(dje): Go objects in structs.  */
-			  write_exp_elt_opcode (UNOP_SIZEOF);
+			  write_exp_elt_opcode (pstate, UNOP_SIZEOF);
 			}
     break;
 
   case 59:
 
 /* Line 1455 of yacc.c  */
-#line 517 "go-exp.y"
+#line 527 "go-exp.y"
     {
 			  /* We copy the string here, and not in the
 			     lexer, to guarantee that we do not leak a
@@ -2125,7 +2135,7 @@ yyreduce:
   case 60:
 
 /* Line 1455 of yacc.c  */
-#line 534 "go-exp.y"
+#line 544 "go-exp.y"
     {
 			  /* Note that we NUL-terminate here, but just
 			     for convenience.  */
@@ -2146,11 +2156,12 @@ yyreduce:
   case 61:
 
 /* Line 1455 of yacc.c  */
-#line 552 "go-exp.y"
+#line 562 "go-exp.y"
     {
 			  int i;
 
-			  write_exp_string_vector (0 /*always utf8*/, &(yyvsp[(1) - (1)].svec));
+			  write_exp_string_vector (pstate, 0 /*always utf8*/,
+						   &(yyvsp[(1) - (1)].svec));
 			  for (i = 0; i < (yyvsp[(1) - (1)].svec).len; ++i)
 			    xfree ((yyvsp[(1) - (1)].svec).tokens[i].ptr);
 			  xfree ((yyvsp[(1) - (1)].svec).tokens);
@@ -2160,25 +2171,25 @@ yyreduce:
   case 62:
 
 /* Line 1455 of yacc.c  */
-#line 563 "go-exp.y"
-    { write_exp_elt_opcode (OP_BOOL);
-			  write_exp_elt_longcst ((LONGEST) (yyvsp[(1) - (1)].lval));
-			  write_exp_elt_opcode (OP_BOOL); }
+#line 574 "go-exp.y"
+    { write_exp_elt_opcode (pstate, OP_BOOL);
+			  write_exp_elt_longcst (pstate, (LONGEST) (yyvsp[(1) - (1)].lval));
+			  write_exp_elt_opcode (pstate, OP_BOOL); }
     break;
 
   case 63:
 
 /* Line 1455 of yacc.c  */
-#line 569 "go-exp.y"
-    { write_exp_elt_opcode (OP_BOOL);
-			  write_exp_elt_longcst ((LONGEST) (yyvsp[(1) - (1)].lval));
-			  write_exp_elt_opcode (OP_BOOL); }
+#line 580 "go-exp.y"
+    { write_exp_elt_opcode (pstate, OP_BOOL);
+			  write_exp_elt_longcst (pstate, (LONGEST) (yyvsp[(1) - (1)].lval));
+			  write_exp_elt_opcode (pstate, OP_BOOL); }
     break;
 
   case 64:
 
 /* Line 1455 of yacc.c  */
-#line 575 "go-exp.y"
+#line 586 "go-exp.y"
     { struct symbol *sym = (yyvsp[(1) - (2)].ssym).sym;
 
 			  if (sym == NULL
@@ -2188,16 +2199,16 @@ yyreduce:
 				     "parameters, not for \"%s\""),
 				   copy_name ((yyvsp[(1) - (2)].ssym).stoken));
 
-			  write_exp_elt_opcode (OP_VAR_ENTRY_VALUE);
-			  write_exp_elt_sym (sym);
-			  write_exp_elt_opcode (OP_VAR_ENTRY_VALUE);
+			  write_exp_elt_opcode (pstate, OP_VAR_ENTRY_VALUE);
+			  write_exp_elt_sym (pstate, sym);
+			  write_exp_elt_opcode (pstate, OP_VAR_ENTRY_VALUE);
 			}
     break;
 
   case 65:
 
 /* Line 1455 of yacc.c  */
-#line 591 "go-exp.y"
+#line 602 "go-exp.y"
     { struct symbol *sym = (yyvsp[(1) - (1)].ssym).sym;
 
 			  if (sym)
@@ -2210,13 +2221,13 @@ yyreduce:
 				    innermost_block = block_found;
 				}
 
-			      write_exp_elt_opcode (OP_VAR_VALUE);
+			      write_exp_elt_opcode (pstate, OP_VAR_VALUE);
 			      /* We want to use the selected frame, not
 				 another more inner frame which happens to
 				 be in the same block.  */
-			      write_exp_elt_block (NULL);
-			      write_exp_elt_sym (sym);
-			      write_exp_elt_opcode (OP_VAR_VALUE);
+			      write_exp_elt_block (pstate, NULL);
+			      write_exp_elt_sym (pstate, sym);
+			      write_exp_elt_opcode (pstate, OP_VAR_VALUE);
 			    }
 			  else if ((yyvsp[(1) - (1)].ssym).is_a_field_of_this)
 			    {
@@ -2232,7 +2243,7 @@ yyreduce:
 			      msymbol =
 				lookup_bound_minimal_symbol (arg);
 			      if (msymbol.minsym != NULL)
-				write_exp_msymbol (msymbol);
+				write_exp_msymbol (pstate, msymbol);
 			      else if (!have_full_symbols ()
 				       && !have_partial_symbols ())
 				error (_("No symbol table is loaded.  "
@@ -2247,29 +2258,29 @@ yyreduce:
   case 66:
 
 /* Line 1455 of yacc.c  */
-#line 646 "go-exp.y"
+#line 657 "go-exp.y"
     { (yyval.tval) = lookup_pointer_type ((yyvsp[(2) - (2)].tval)); }
     break;
 
   case 67:
 
 /* Line 1455 of yacc.c  */
-#line 648 "go-exp.y"
+#line 659 "go-exp.y"
     { (yyval.tval) = (yyvsp[(1) - (1)].tsym).type; }
     break;
 
   case 68:
 
 /* Line 1455 of yacc.c  */
-#line 655 "go-exp.y"
-    { (yyval.tval) = builtin_go_type (parse_gdbarch)
+#line 666 "go-exp.y"
+    { (yyval.tval) = builtin_go_type (parse_gdbarch (pstate))
 			    ->builtin_uint8; }
     break;
 
 
 
 /* Line 1455 of yacc.c  */
-#line 2274 "go-exp.c"
+#line 2285 "go-exp.c"
       default: break;
     }
   YY_SYMBOL_PRINT ("-> $$ =", yyr1[yyn], &yyval, &yyloc);
@@ -2481,7 +2492,7 @@ yyreturn:
 
 
 /* Line 1675 of yacc.c  */
-#line 677 "go-exp.y"
+#line 688 "go-exp.y"
 
 
 /* Wrapper on parse_c_float to get the type right for Go.  */
@@ -2512,7 +2523,8 @@ parse_go_float (struct gdbarch *gdbarch, const char *p, int len,
    as our YYSTYPE is different than c-exp.y's  */
 
 static int
-parse_number (const char *p, int len, int parsed_float, YYSTYPE *putithere)
+parse_number (struct parser_state *par_state,
+	      const char *p, int len, int parsed_float, YYSTYPE *putithere)
 {
   /* FIXME: Shouldn't these be unsigned?  We don't deal with negative values
      here, and we do kind of silly things like cast to unsigned.  */
@@ -2537,7 +2549,7 @@ parse_number (const char *p, int len, int parsed_float, YYSTYPE *putithere)
 
   if (parsed_float)
     {
-      if (! parse_go_float (parse_gdbarch, p, len,
+      if (! parse_go_float (parse_gdbarch (par_state), p, len,
 			    &putithere->typed_val_float.dval,
 			    &putithere->typed_val_float.type))
 	return ERROR;
@@ -2653,9 +2665,10 @@ parse_number (const char *p, int len, int parsed_float, YYSTYPE *putithere)
 
   un = (ULONGEST)n >> 2;
   if (long_p == 0
-      && (un >> (gdbarch_int_bit (parse_gdbarch) - 2)) == 0)
+      && (un >> (gdbarch_int_bit (parse_gdbarch (par_state)) - 2)) == 0)
     {
-      high_bit = ((ULONGEST)1) << (gdbarch_int_bit (parse_gdbarch) - 1);
+      high_bit
+        = ((ULONGEST)1) << (gdbarch_int_bit (parse_gdbarch (par_state)) - 1);
 
       /* A large decimal (not hex or octal) constant (between INT_MAX
 	 and UINT_MAX) is a long or unsigned long, according to ANSI,
@@ -2663,28 +2676,29 @@ parse_number (const char *p, int len, int parsed_float, YYSTYPE *putithere)
 	 int.  This probably should be fixed.  GCC gives a warning on
 	 such constants.  */
 
-      unsigned_type = parse_type->builtin_unsigned_int;
-      signed_type = parse_type->builtin_int;
+      unsigned_type = parse_type (par_state)->builtin_unsigned_int;
+      signed_type = parse_type (par_state)->builtin_int;
     }
   else if (long_p <= 1
-	   && (un >> (gdbarch_long_bit (parse_gdbarch) - 2)) == 0)
+	   && (un >> (gdbarch_long_bit (parse_gdbarch (par_state)) - 2)) == 0)
     {
-      high_bit = ((ULONGEST)1) << (gdbarch_long_bit (parse_gdbarch) - 1);
-      unsigned_type = parse_type->builtin_unsigned_long;
-      signed_type = parse_type->builtin_long;
+      high_bit
+	= ((ULONGEST)1) << (gdbarch_long_bit (parse_gdbarch (par_state)) - 1);
+      unsigned_type = parse_type (par_state)->builtin_unsigned_long;
+      signed_type = parse_type (par_state)->builtin_long;
     }
   else
     {
       int shift;
       if (sizeof (ULONGEST) * HOST_CHAR_BIT
-	  < gdbarch_long_long_bit (parse_gdbarch))
+	  < gdbarch_long_long_bit (parse_gdbarch (par_state)))
 	/* A long long does not fit in a LONGEST.  */
 	shift = (sizeof (ULONGEST) * HOST_CHAR_BIT - 1);
       else
-	shift = (gdbarch_long_long_bit (parse_gdbarch) - 1);
+	shift = (gdbarch_long_long_bit (parse_gdbarch (par_state)) - 1);
       high_bit = (ULONGEST) 1 << shift;
-      unsigned_type = parse_type->builtin_unsigned_long_long;
-      signed_type = parse_type->builtin_long_long;
+      unsigned_type = parse_type (par_state)->builtin_unsigned_long_long;
+      signed_type = parse_type (par_state)->builtin_long_long;
     }
 
    putithere->typed_val_int.val = n;
@@ -2852,7 +2866,7 @@ static int last_was_structop;
 /* Read one token, getting characters through lexptr.  */
 
 static int
-lex_one_token (void)
+lex_one_token (struct parser_state *par_state)
 {
   int c;
   int namelen;
@@ -2983,7 +2997,8 @@ lex_one_token (void)
 				  && (*p < 'A' || *p > 'Z')))
 	      break;
 	  }
-	toktype = parse_number (tokstart, p - tokstart, got_dot|got_e, &yylval);
+	toktype = parse_number (par_state, tokstart, p - tokstart,
+				got_dot|got_e, &yylval);
         if (toktype == ERROR)
 	  {
 	    char *err_copy = (char *) alloca (p - tokstart + 1);
@@ -3239,7 +3254,7 @@ classify_packaged_name (const struct block *block)
    The result is one of NAME, NAME_OR_INT, or TYPENAME.  */
 
 static int
-classify_name (const struct block *block)
+classify_name (struct parser_state *par_state, const struct block *block)
 {
   struct type *type;
   struct symbol *sym;
@@ -3249,8 +3264,9 @@ classify_name (const struct block *block)
   copy = copy_name (yylval.sval);
 
   /* Try primitive types first so they win over bad/weird debug info.  */
-  type = language_lookup_primitive_type_by_name (parse_language,
-						 parse_gdbarch, copy);
+  type = language_lookup_primitive_type_by_name (parse_language (par_state),
+						 parse_gdbarch (par_state),
+						 copy);
   if (type != NULL)
     {
       /* NOTE: We take advantage of the fact that yylval coming in was a
@@ -3306,7 +3322,8 @@ classify_name (const struct block *block)
       || (copy[0] >= 'A' && copy[0] < 'A' + input_radix - 10))
     {
       YYSTYPE newlval;	/* Its value is ignored.  */
-      int hextype = parse_number (copy, yylval.sval.length, 0, &newlval);
+      int hextype = parse_number (par_state, copy, yylval.sval.length,
+				  0, &newlval);
       if (hextype == INT)
 	{
 	  yylval.ssym.sym = NULL;
@@ -3339,7 +3356,7 @@ yylex (void)
     }
   popping = 0;
 
-  current.token = lex_one_token ();
+  current.token = lex_one_token (pstate);
 
   /* TODO: Need a way to force specifying name1 as a package.
      .name1.name2 ?  */
@@ -3350,14 +3367,14 @@ yylex (void)
   /* See if we have "name1 . name2".  */
 
   current.value = yylval;
-  next.token = lex_one_token ();
+  next.token = lex_one_token (pstate);
   next.value = yylval;
 
   if (next.token == '.')
     {
       token_and_value name2;
 
-      name2.token = lex_one_token ();
+      name2.token = lex_one_token (pstate);
       name2.value = yylval;
 
       if (name2.token == NAME)
@@ -3396,16 +3413,23 @@ yylex (void)
 
   popping = 1;
   yylval = current.value;
-  return classify_name (expression_context_block);
+  return classify_name (pstate, expression_context_block);
 }
 
 int
-go_parse (void)
+go_parse (struct parser_state *par_state)
 {
   int result;
-  struct cleanup *back_to = make_cleanup (null_cleanup, NULL);
+  struct cleanup *back_to;
+
+  /* Setting up the parser state.  */
+  gdb_assert (par_state != NULL);
+  pstate = par_state;
+
+  back_to = make_cleanup (null_cleanup, NULL);
 
   make_cleanup_restore_integer (&yydebug);
+  make_cleanup_clear_parser_state (&pstate);
   yydebug = parser_debug;
 
   /* Initialize some state used by the lexer.  */
